/*
 * Base64 Encode
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.2 $  $Date: 2003/04/13 22:43:37 $
 */

#include <phlib/xl_config.h>
#include <phlib/xl_types.h>
#include <phlib/xl_base64_encode.h>

/* Encode table. Very straightforward..... 
 */
static const unsigned char base_64_encode_table[65]= \
"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

/* ------------------------------------------------------------------------- */
/* Encode to base 64. Nuff said... 
 */ 
size_t 
xl_Base64_encode( void *dst, const void *src, size_t size)
{
  const size_t                  encoded = ((size + 2) / 3) * 4;
  register const unsigned char *in      = (const unsigned char *)src;
  register unsigned char       *out     = (unsigned char *)dst;

  /* For every 3 bytes, expand to 4 chars... */
  while( size > 2 ) {
    *(out++)=base_64_encode_table[in[0]>>2];
    *(out++)=base_64_encode_table[((in[0]&3)<<4)+(in[1]>>4)];
    *(out++)=base_64_encode_table[((in[1]&15)<<2)+(in[2]>>6)];
    *(out++)=base_64_encode_table[in[2]&63];
    in+=3;
    size-=3;
  }

  /* Leftover characters padded with ='s */
  if( size == 2 ) {
    *(out++)=base_64_encode_table[in[0]>>2];
    *(out++)=base_64_encode_table[((in[0]&3)<<4)+(in[1]>>4)];
    *(out++)=base_64_encode_table[((in[1])&15)<<2];
    *out='=';
  }
  else if( size == 1 ) {
    *(out++)=base_64_encode_table[in[0]>>2];
    *(out++)=base_64_encode_table[(in[0]&3)<<4];
    *(out++)='=';
    *out='=';
  }
  return encoded;
}

/* ------------------------------------------------------------------------- */
/* end */
