/*
 * Code to handle directory stuff
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.3 $  $Date: 2003/04/13 22:43:37 $
 * ---------------------------------------------------------------------- 
 */

#include <phlib/xl_config.h>
#include <string.h> 
#if defined(__FreeBSD__) || (defined(__APPLE__) && defined(__MACH__))
#  include <stdlib.h> /* For memory functions */
#else
#  include <malloc.h> /* For memory functions */
#endif

#if HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# if HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# if HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# if HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif


#include <phlib/xl_types.h>
#include <phlib/xl_dir.h>

/* ----------------------------------------------------------------------- */

/* 
 *returns a void prt to open directory 
 */
void *
xl_open_dir( const char *dpath )
{
  DIR *dp;
  RetValue ret = Success;

  if( (dp = opendir( dpath )) == NULL ) ret = Failure;
  
  return (void*)dp;
}

/* ----------------------------------------------------------------------- */

void
xl_close_dir( void* dp )
{
  closedir( (DIR*)dp );
}

/* ----------------------------------------------------------------------- */
/*
 * Returns a pointer the filename
 * NULL and errno == 0 at end of list
 * NULL and errno != 0 in error
 */
char *
xl_get_dir_entry( void* dp )
{
  struct dirent *dir;
  char   *name = NULL;
  int nl;

  while( (dir = readdir( (DIR*)dp)) != NULL ) {
    if( dir->d_ino == 0 ) continue; /* Skip removed files */
    name = dir->d_name;
    nl = NAMLEN(dir);
    name[nl] = '\0'; /* Make sure string is terminated */
    if( nl == 1 && name[0] == '.' ) continue;
    else if( nl == 2 && name[1] == '.' && name[0] == '.' ) continue;
    break; /* leave loop.  The only point of it is to skip removed files */
  }
  return name;
}

/* ----------------------------------------------------------------------- */
/* end */
